<?php

/**
 * UGCC WHMCS Provisioning Module
 *  https://www.brainless.us
 * @copyright Copyright (c) 2020 Brainless Technologies
 * @license Proprietary Confidential
 * 1.0 -- Initial Release
 * 1.1 -- Added CPU Priority and MAX Ram Support
 */
if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}

function ugccapi_alive(string $url, string $uname, string $pw) {
    if (!$homepage = file_get_contents($url . "/api.ashx?u=" . urlencode($uname) . "&p=" . urlencode($pw) . "&c=alive", "w")) {
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        return true;
    }
    return false;
}

function ugccapi_createuser(string $url, string $serveruname, string $serverpw, string $acctname, string $password, string $email, string &$error) {
    if ($acctname === "") {
        $acctname = $email;
    }
if (!$homepage = file_get_contents($url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=newuser&o1=" . urlencode($acctname) . "&o2=" . urlencode($email) . "&o3=" . urlencode($password), "w")) {
        $error = "Unable to connect to UGCC API";
        return false;
    }
    if (strpos($homepage, "OK") !== false || strpos($homepage, "already exists") != false) {
        //Return true on successful new user addition, also return true if user already exists
        return true;
    }
    $error = $homepage;
    return false;
}

function ugccapi_install(string $url, string $serveruname, string $serverpw, string $UID, string $slots, $loc, $inst1, $inst2, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=installfastloc&i=" . $UID . "&o1=" . $inst1 . ',' . $inst2 . "&o2=" . urlencode($loc) . "&o3=" . $slots;

    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('install', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        //Return True if we get an OK, pass new server ID in $error
        $homepage2 = str_replace("\n", ":", $homepage);
        $homepage2 = str_replace("OK:  ,", '', $homepage2);
        $homepage2 = trim($homepage2, ':');
        $users = explode(":", $homepage2);
        $error = trim($users[3]);
        ugcc_logModCalls('install', $requesturl, $homepage, $error, $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('install', $requesturl, $homepage, $error, $serverpw);
    return false;
}

function ugccapi_resetpw(string $url, string $serveruname, string $serverpw, string $acctid, string $password, string $email, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=pw" . "&i=" . $acctid . "&o1=" . urlencode($password);
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('resetpw', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('resetpw', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('resetpw', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_disableUser(string $url, string $serveruname, string $serverpw, string $UID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=disableu" . "&i=" . $UID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('disableUser', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('disableUser', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('disableUser', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_enableUser(string $url, string $serveruname, string $serverpw, string $UID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=enableu" . "&i=" . $UID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('enableUser', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('enableUser', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('enableUser', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_disableServer(string $url, string $serveruname, string $serverpw, string $SID, string &$error, string $comment, bool $disableIfNoMoreServers) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=disables" . "&i=" . $SID;
    if (!empty($comment)) {
           $requesturl = $requesturl . "&o1=" . urlencode($comment);
        }
        if ($disableIfNoMoreServers)
        {
           $requesturl = $requesturl . "&o2=1"; 
        }
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('disableServer', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('disableServer', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('disableServer', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_enableServer(string $url, string $serveruname, string $serverpw, string $SID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=enables" . "&i=" . $SID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('enableServer', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('enableServer', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('enableServer', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_updateDbServer(string $url, string $serveruname, string $serverpw, string $SID, string $column, string $data, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=updatedbserver" . "&i=" . $SID . "&o1=" . urlencode($column) . "&o2=" . urlencode($data);
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('updateDBServer', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('updateDBServer', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('updateDBServer', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_serverStatus(string $url, string $serveruname, string $serverpw, string $SID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=status" . "&i=" . $SID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('serverStatus', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        if (strpos($homepage, "Running"))
        {
                $error = "running";
        }
        if (strpos($homepage, "Stopped"))
        {
                $error='stopped';
        }
        if (strpos($homepage, "Updating"))
        {
                $error='updating';
        }
        ugcc_logModCalls('serverStatus', $requesturl, $homepage, $error, $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('serverStatus', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_startServer(string $url, string $serveruname, string $serverpw, string $SID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=start" . "&i=" . $SID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('startServer', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('startServer', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('startServer', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugccapi_stopServer(string $url, string $serveruname, string $serverpw, string $SID, string &$error) {
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=stop" . "&i=" . $SID;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('stopServer', $requesturl, '', $error, $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        ugcc_logModCalls('stopServer', $requesturl, $homepage, 'success', $serverpw);
        return true;
    }
    $error = $homepage;
    ugcc_logModCalls('stopServer', $requesturl, $homepage, 'error', $serverpw);
    return false;
}
function ugcc_logModCalls($action, $requestString, $responseData, $processedData, $mask) {
    $pwmask[0] = $mask;
    $pwmask[1] = urlencode($mask);
    logModuleCall('ugcc', $action, $requestString, $responseData, $processedData, $pwmask);
}

function ugccapi_getuid(string $url, string $serveruname, string $serverpw, string $acctname, string $password, string $email, string &$error) {
    if ($acctname === "") {
        $acctname = $email;
    }
    $requesturl = $url . "/api.ashx?u=" . urlencode($serveruname) . "&p=" . urlencode($serverpw) . "&c=listusers";
    if (!$homepage = file_get_contents($requesturl, "w")) {
        $error = "Unable to connect to UGCC API";
        ugcc_logModCalls('getuid', $requesturl, $error, 'error', $serverpw);
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        //id, username, email
        $homepage2 = str_replace("\n", ",", $homepage);
        $homepage2 = str_replace("OK:  ", ",", $homepage2);
        $homepage2 = trim($homepage2, ',');
        $users = explode(",", $homepage2);
        for ($i = 4; $i <= count($users); $i += 3) {
            if (ugccapi_uncsvquote($users[$i]) === $acctname) {
                $error = ugccapi_uncsvquote($users[$i - 1]);
                ugcc_logModCalls("getuid", $requesturl, $homepage, $error, $serverpw);
                return true;
            }
        }
        ugcc_logModCalls("getuid", $requesturl, $homepage, "User not found", $serverpw);
        $error = 'User not found';
        return false;
    }
    ugcc_logModCalls("getuid", $requesturl, $homepage, 'error', $serverpw);
    $error = $homepage;
    return false;
}

function ugccapi_uncsvquote($data) {
    $data = str_replace('""', '"', $data);
    $data = trim($data, '"');
    return $data;
}

function ugccapi_getInstallerIds(array $params) {
    //Loader function that loads installer IDs into module settings
    $myarray = array();
    $requesturl = $params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'] . "/api.ashx?u=" . urlencode($params['serverusername']) . "&p=" . urlencode($params['serverpassword']) . "&c=listinstallers";
    if (!$homepage = file_get_contents($requesturl, "w")) {
        ugcc_logModCalls("getinstallerids", $requesturl, '', 'error', $params['serverpassword']);
        throw new Exception('Empty response or unable to connect to API');
         
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        //id, name, gametype, platform
        $homepage2 = str_replace("\n", ",", $homepage);
        $homepage2 = str_replace("OK:  ", ",", $homepage2);
        $homepage2 = trim($homepage2, ',');
        $users = explode(",", $homepage2);
        for ($i = 4; $i < count($users); $i += 4) {
            $myarray[$users[$i]] = $users[$i + 1] . ' (' . $users[$i + 2] . ', ' . $users[$i + 3] . ')';
        }
    } else {
        throw new Exception('Unexpected response received from API');
    }
    return $myarray;
}
function ugccapi_getServerDetails(array $params, &$error) {
    //Loader function that loads installer IDs into module settings
    $myarray = array();
    $requesturl = $params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'] . "/api.ashx?u=" . urlencode($params['serverusername']) . "&p=" . urlencode($params['serverpassword']) . "&c=serverinfo&i=" .$params['customfields']['SID'] ;
    if (!$homepage = file_get_contents($requesturl, "w")) {
        ugcc_logModCalls("getServerDetails", $requesturl, '', 'error', $params['serverpassword']);
        //throw new Exception('Empty response or unable to connect to API');
         $error = 'Unable to connect to API';
        return false;
    }
    if (strpos($homepage, "OK") !== false) {
        //id, name, gametype, platform
        //$homepage2 = str_replace("\n", ",", $homepage);
        $homepage2 = str_replace("OK:  ", "", $homepage);
        $homepage2 = trim($homepage2, ' ');
        //$users = explode(",", $homepage2);
       $error = $homepage2;
       return true;
    } else {
        $error = $homepage;
        return false;
    }
    return false;
}
// Require any libraries needed for the module to function.
// require_once __DIR__ . '/path/to/library/loader.php';
//
// Also, perform any initialization required by the service's library.

/**
 * Define module related meta data.
 *
 * Values returned here are used to determine module related abilities and
 * settings.
 *
 * @see https://developers.whmcs.com/provisioning-modules/meta-data-params/
 *
 * @return array
 */
function ugcc_MetaData() {
    return array(
        'DisplayName' => 'UGCC Provisioning Module',
        'APIVersion' => '1.1', // Use API Version 1.1
        'RequiresServer' => true, // Set true if module requires a server to work
        'DefaultNonSSLPort' => '80', // Default Non-SSL Connection Port
        'DefaultSSLPort' => '443', // Default SSL Connection Port
        'ServiceSingleSignOnLabel' => 'UGCC User Login',
        'AdminSingleSignOnLabel' => 'UGCC Admin Login',
    );
}

/**
 * Define product configuration options.
 *
 * The values you return here define the configuration options that are
 * presented to a user when configuring a product for use with the module. These
 * values are then made available in all module function calls with the key name
 * configoptionX - with X being the index number of the field from 1 to 24.
 *
 * You can specify up to 24 parameters, with field types:
 * * text
 * * password
 * * yesno
 * * dropdown
 * * radio
 * * textarea
 *
 * Examples of each and their possible configuration parameters are provided in
 * this sample function.
 *
 * @see https://developers.whmcs.com/provisioning-modules/config-options/
 *
 * @return array
 */
function ugcc_ConfigOptions() {
    return array(
        // a text field type allows for single line text input
        'Installer ID Win' => array(
            'Type' => 'text',
            'Size' => '25',
            'Loader' => 'ugccapi_getInstallerIds',
            'SimpleMode' => true,
            'Default' => '0',
            'Description' => 'Enter ID of auto installer used in UGCC for Windows',
        ),
        // a text field type allows for single line text input
        'Installer ID Linux' => array(
            'Type' => 'text',
            'Size' => '25',
            'Loader' => 'ugccapi_getInstallerIds',
            'SimpleMode' => true,
            'Default' => '0',
            'Description' => 'Enter ID of auto installer used in UGCC for Linux',
        ),
        'Base Slots' => array(
            'Type' => 'text',
            'Size' => '3',
            'Default' => '0',
            'SimpleMode' => true,
            'Description' => 'Base Slot Count for this product.  Any additional slots a customer adds will be added to this value to determine max slots.  Enter a number.',
        ),
//         'Start Server' => array(
//            'Type' => 'yesno',
//            'Description' => 'Start server after deployment',
//        ),
//        'Email Server Info' => array(
//            'Type' => 'yesno',
//            'Description' => 'Send email once deployment is finished',
//            'SimpleMode' => true,
//        ),
//        'Loction ID' => array(
//            'Type' => 'text',
//            'Size' => '3',
//            'Default' => '0',
//            'Description' => 'Enter ID of location used in UGCC',
//        ),
    );
}

/**
 * Provision a new instance of a product/service.
 *
 * Attempt to provision a new instance of a given product/service. This is
 * called any time provisioning is requested inside of WHMCS. Depending upon the
 * configuration, this can be any of:
 * * When a new order is placed
 * * When an invoice for a new order is paid
 * * Upon manual request by an admin user
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function ugcc_CreateAccount(array $params) {
    //ugcc_logModCalls("CreateAccount", )
    try {
        if (!empty($params['customfields']['SID'])) {
            return "Server already setup";
        }

        if (empty($params['username'])) {
            //If username is blank, set it to user's email address.  We need to prevent users from choosing their own usernames; their email should be unique enough
            $params['model']->serviceProperties->save(['Username' => $params['clientsdetails']['email']]);
        }
        if ( empty($params['customfields']['GCP Password'])===false && ! ($params['customfields']['GCP Password'] === $params["password"]))
        {
            //Set WHMCS password to user specified password
            $params['model']->serviceProperties->save(['Password' => $params['customfields']['GCP Password'] ]);
            
        }
        $error = "";
        $success = ugccapi_createuser($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['username'], $params['password'], $params['clientsdetails']['email'], $error);
        if (!$success) {
            return $error;
        }
        
        $success = ugccapi_getuid($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['username'], $params['password'], $params['clientsdetails']['email'], $error);
        if (!$success) {
            return $error;
        }
        //Update custom UID field
        $params['model']->serviceProperties->save(['UID' => $error]);
        $UID = $error;
        
        //Ensure account is active, if the account already exists and a past customer, re-enable account
        $success = ugccapi_enableUser($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $UID, $error);
        if (!$success) {
            return $error;
        }

//ugccapi_install($error, $serveruname, $serverpw, $UID, $slots, $loc, $inst1, $inst2, $error)
        $baseslots = $params["configoption3"];
        $addslots = $params["configoptions"]["Additional Player Slots"];
       $maxslots = $baseslots + $addslots;
        $success = ugccapi_install($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $UID, $maxslots, $params["customfields"]["Location"], $params["configoption1"], $params["configoption2"], $error);
        if (!$success) {
            return $error;
        }
        //Update custom SID field with ID of new server from API
        $params['model']->serviceProperties->save(['SID' => $error]);

       $SID = $error;
        //Example using API to change server settings
        $priority = $params["configoptions"]["Server CPU Priority"];
        if (!empty($priority) && $priority !== "Normal")
        {
            $priorityvalue=4;
            //db field is 'priority', values are 5 = realtime, 2=high, 4=normal
            if (strtolower($priority) === 'realtime')
            {
                $priorityvalue = 5;
            }
            if (strtolower($priority) === 'high')
            {
                $priorityvalue = 2;
            }
            $success = ugccapi_updateDbServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $SID, 'priority', $priorityvalue, $error);
            if (!$success) {
                return $error;
            }
        }
        
        $maxram = $params["configoptions"]["RAM Allowance"];
        if (!empty(maxram))
        {
            //db field is 'ramthreshold', expected value should be in MB
            $multipler = 1; //For MB, default if no match is found
            //Get rid of any spaces
            $maxram = str_replace(' ', '', $maxram);
            $maxram = strtolower($maxram);
            //Check text to figure out which multiplier to use
            $position = strpos($maxram,'gb');
            if ($position !== false) {
                //GB, so multiply value by 1024
                $multipler = 1024;
            }
            $position = strpos($maxram,'tb');
            if ($position !== false) {
                //TB, so multiply value by 1024 * 1024; maybe someday we'll have TB's of ram :)
                $multipler = 1024 * 1024;
            }
            $maxram = substr($maxram, 0, strlen($maxram) - 2);
            $maxram = $maxram * $multipler;
            $success = ugccapi_updateDbServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $SID, 'ramthreshold', $maxram, $error);
            if (!$success) {
                return $error;
            }
        }
        // Utilise Service Properties to retrieve the Order Number
//        $amount = $params['model']->serviceProperties->get('Amount');
//        $success = ugccapi_updateDbServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $SID, 'amountdue', $amount, $error);
//            if (!$success) {
//                return $error;
//            }
        return 'success';
        //return $error;
        //Install Server
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Suspend an instance of a product/service.
 *
 * Called when a suspension is requested. This is invoked automatically by WHMCS
 * when a product becomes overdue on payment or can be called manually by admin
 * user.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function ugcc_SuspendAccount(array $params) {
    //Disables a server, not the user's account.  User could have multiple products
     if (empty($params['customfields']['SID'])) {
           return "No server has been setup yet, unable to suspend";
        }
    try {
        $error = "";
        $success = ugccapi_disableServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error,"WHMCS Suspend: " . $params["suspendreason"],false);
        if (!$success) {
            return $error;
        }
        //Blindly stop the server
        ugccapi_stopServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Un-suspend instance of a product/service.
 *
 * Called when an un-suspension is requested. This is invoked
 * automatically upon payment of an overdue invoice for a product, or
 * can be called manually by admin user.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function ugcc_UnsuspendAccount(array $params) {
    //Re-enables a server, not the user's account.  User could have multiple products
         if (empty($params['customfields']['SID'])) {
            return "No server has been setup yet, unable to unsuspend";
        }
    try {
         $error = "";
        $success = ugccapi_enableServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
        if (!$success) {
            return $error;
        }
        //Blindly start the server
        ugccapi_startServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );
        return $e->getMessage();
    }
    return 'success';
}

/**
 * Terminate instance of a product/service.
 *
 * Called when a termination is requested. This can be invoked automatically for
 * overdue products if enabled, or requested manually by an admin user.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function ugcc_TerminateAccount(array $params) {
       //Disables a server, not the user's account.  User could have multiple products
     if (empty($params['customfields']['SID'])) {
           return "No server has been setup yet, unable to suspend";
        }
    try {
        $error = "";
        $success = ugccapi_disableServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error,"WHMCS Terminate",true);
        if (!$success) {
            return $error;
        }
        //Blindly stop the server
        ugccapi_stopServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    return 'success';
}

/**
 * Change the password for an instance of a product/service.
 *
 * Called when a password change is requested. This can occur either due to a
 * client requesting it via the client area or an admin requesting it from the
 * admin side.
 *
 * This option is only available to client end users when the product is in an
 * active status.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
function ugcc_ChangePassword(array $params) {
    try {
        $error = "";
        if (empty($params['customfields']["UID"]))
        {
            return 'UID needs to be set, run create first';
        }
                
        $success = ugccapi_resetpw($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["UID"], $params['password'], $params['clientsdetails']['email'], $error);
        if (!$success) {
            return $error;
        }
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}

/**
 * Upgrade or downgrade an instance of a product/service.
 *
 * Called to apply any change in product assignment or parameters. It
 * is called to provision upgrade or downgrade orders, as well as being
 * able to be invoked manually by an admin user.
 *
 * This same function is called for upgrades and downgrades of both
 * products and configurable options.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return string "success" or an error message
 */
//function ugcc_ChangePackage(array $params) {
//    try {
//        // Call the service's change password function, using the values
//        // provided by WHMCS in `$params`.
//        //
//        // A sample `$params` array may be defined as:
//        //
//        // ```
//        // array(
//        //     'username' => 'The service username',
//        //     'configoption1' => 'The new service disk space',
//        //     'configoption3' => 'Whether or not to enable FTP',
//        // )
//        // ```
//    } catch (Exception $e) {
//        // Record the error in WHMCS's module log.
//        logModuleCall(
//                'ugcc',
//                __FUNCTION__,
//                $params,
//                $e->getMessage(),
//                $e->getTraceAsString()
//        );
//
//        return $e->getMessage();
//    }
//
//    return 'success';
//}

/**
 * Test connection with the given server parameters.
 *
 * Allows an admin user to verify that an API connection can be
 * successfully made with the given configuration parameters for a
 * server.
 *
 * When defined in a module, a Test Connection button will appear
 * alongside the Server Type dropdown when adding or editing an
 * existing server.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function ugcc_TestConnection(array $params) {
    try {
        // Call the service's connection test function.
        $success = ugccapi_alive($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword']);
        //$success = true;
        //$errorMsg = '';
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }

    return array(
        'success' => $success,
        'error' => $errorMsg,
    );
}

/**
 * Additional actions an admin user can invoke.
 *
 * Define additional actions that an admin user can perform for an
 * instance of a product/service.
 *
 * @see provisioningmodule_buttonOneFunction()
 *
 * @return array
 */
function ugcc_AdminCustomButtonArray() {
    return array(
        "Stop Server" => "adminStopButtonFunction",
        "Start Server" => "adminStartButtonFunction",
    );
}

/**
 * Additional actions a client user can invoke.
 *
 * Define additional actions a client user can perform for an instance of a
 * product/service.
 *
 * Any actions you define here will be automatically displayed in the available
 * list of actions within the client area.
 *
 * @return array
 */
function ugcc_ClientAreaCustomButtonArray() {
    return array(
        "Start Server" => "clientStartServerFunction",
        "Stop Server" => "clientStopServerFunction",
    );
}

/**
 * Custom function for performing an additional action.
 *
 * You can define an unlimited number of custom functions in this way.
 *
 * Similar to all other module call functions, they should either return
 * 'success' or an error message to be displayed.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see provisioningmodule_AdminCustomButtonArray()
 *
 * @return string "success" or an error message
 */
function ugcc_adminStopButtonFunction(array $params) {
    try {
        $error = "";
        $success = ugccapi_stopServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
        if (!$success) {
            return $error;
        }
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}
function ugcc_adminStartButtonFunction(array $params) {
    try {
        $error = "";
        $success = ugccapi_startServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
        if (!$success) {
            return $error;
        }
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}
/**
 * Custom function for performing an additional action.
 *
 * You can define an unlimited number of custom functions in this way.
 *
 * Similar to all other module call functions, they should either return
 * 'success' or an error message to be displayed.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see provisioningmodule_ClientAreaCustomButtonArray()
 *
 * @return string "success" or an error message
 */
function ugcc_clientStartServerFunction(array $params) {
    try {
        $username = $params['username'];
        if ($username === "") {
            $username = $params['clientsdetails']['email'];
        }
         $error = "";
        $success = ugccapi_startServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $username, $params['password'], $params['customfields']["SID"], $error); 
        if (!$success) {
            return $error;
        }
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}
function ugcc_clientStopServerFunction(array $params) {
    try {
        $username = $params['username'];
        if ($username === "") {
            $username = $params['clientsdetails']['email'];
        }
         $error = "";
        $success = ugccapi_stopServer($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $username, $params['password'], $params['customfields']["SID"], $error); 
        if (!$success) {
            return $error;
        }
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return $e->getMessage();
    }

    return 'success';
}
/**
 * Admin services tab additional fields.
 *
 * Define additional rows and fields to be displayed in the admin area service
 * information and management page within the clients profile.
 *
 * Supports an unlimited number of additional field labels and content of any
 * type to output.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see provisioningmodule_AdminServicesTabFieldsSave()
 *
 * @return array
 */
function ugcc_AdminServicesTabFields(array $params) {
    try {
        
        $error = "";
        $status = "";
        if (!empty($params['customfields']['SID']))
        {
            $success = ugccapi_serverStatus($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
            if (!$success) {
                $status = $error;
            }
            else
            {
                if ($error === 'running')
                {
                    $status = '<font color="green"><b>Running</b></font>';
                }
                if ($error === 'stopped')
                {
                    $status = '<font color="red"><b>Stopped</b></font>';
                }
                if ($error === 'updating')
                {
                    $status = '<font color="orange"><b>Updating</b></font>';
                }
                if (empty($status))
                {
                    $status = $error;
                }
            }
            $success = ugccapi_getServerDetails($params,$error);
            if (!$success) {
                $details = "Unable to connect to API";
            }
            $details = $error;
            }
        else
        {
            $status = "Server not yet created";
            $details = $status;
        } 
        
        $response = array();
        // Return an array based on the function's response.
        return array(
            'Server Status' => (string)$status,
            'Server Details' => $details,
            //'Number of Oranges' => (int) $response['numOranges'],
            //'Last Access Date' => date("Y-m-d H:i:s", $response['lastLoginTimestamp']),
            //'Something Editable' => '<input type="hidden" name="ugcc_original_uniquefieldname" '
            //. 'value="' . htmlspecialchars($response['textvalue']) . '" />'
            //. '<input type="text" name="ugcc_uniquefieldname"'
            //. 'value="' . htmlspecialchars($response['textvalue']) . '" />',
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        // In an error condition, simply return no additional fields to display.
    }

    return array();
}

/**
 * Execute actions upon save of an instance of a product/service.
 *
 * Use to perform any required actions upon the submission of the admin area
 * product management form.
 *
 * It can also be used in conjunction with the AdminServicesTabFields function
 * to handle values submitted in any custom fields which is demonstrated here.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 * @see provisioningmodule_AdminServicesTabFields()
 */
function ugcc_AdminServicesTabFieldsSave(array $params) {
    // Fetch form submission variables.
    //$originalFieldValue = isset($_REQUEST['ugcc_original_uniquefieldname']) ? $_REQUEST['ugcc_original_uniquefieldname'] : '';

    //$newFieldValue = isset($_REQUEST['ugcc_uniquefieldname']) ? $_REQUEST['ugcc_uniquefieldname'] : '';

    // Look for a change in value to avoid making unnecessary service calls.
    //if ($originalFieldValue != $newFieldValue) {
    //    try {
    //        // Call the service's function, using the values provided by WHMCS
    //        // in `$params`.
    //    } catch (Exception $e) {
    //        // Record the error in WHMCS's module log.
    //        logModuleCall(
    //                'ugcc',
    //                __FUNCTION__,
    //                $params,
    //                $e->getMessage(),
    //                $e->getTraceAsString()
    //        );

            // Otherwise, error conditions are not supported in this operation.
    //    }
    //}
}

/**
 * Perform single sign-on for a given instance of a product/service.
 *
 * Called when single sign-on is requested for an instance of a product/service.
 *
 * When successful, returns a URL to which the user should be redirected.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function ugcc_ServiceSingleSignOn(array $params) {
    try {
        $response = array();
        $username = $params['username'];
        if ($username === "") {
            $username = $params['clientsdetails']['email'];
        }
        $response['redirectUrl'] = $params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'] . "/Default.aspx?user=" . urlencode($username) . "&password=" . urlencode($params['password']);
        return array(
            'success' => true,
            'redirectTo' => $response['redirectUrl'],
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}

/**
 * Perform single sign-on for a server.
 *
 * Called when single sign-on is requested for a server assigned to the module.
 *
 * This differs from ServiceSingleSignOn in that it relates to a server
 * instance within the admin area, as opposed to a single client instance of a
 * product/service.
 *
 * When successful, returns a URL to which the user should be redirected to.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function ugcc_AdminSingleSignOn(array $params) {
    try {
        // Call the service's single sign-on admin token retrieval function,
        // using the values provided by WHMCS in `$params`.
        $response = array();
        $response['redirectUrl'] = $params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'] . "/Default.aspx?user=" . urlencode($params['serverusername']) . "&password=" . urlencode($params['serverpassword']);
        return array(
            'success' => true,
            'redirectTo' => $response['redirectUrl'],
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        return array(
            'success' => false,
            'errorMsg' => $e->getMessage(),
        );
    }
}

/**
 * Client area output logic handling.
 *
 * This function is used to define module specific client area output. It should
 * return an array consisting of a template file and optional additional
 * template variables to make available to that template.
 *
 * The template file you return can be one of two types:
 *
 * * tabOverviewModuleOutputTemplate - The output of the template provided here
 *   will be displayed as part of the default product/service client area
 *   product overview page.
 *
 * * tabOverviewReplacementTemplate - Alternatively using this option allows you
 *   to entirely take control of the product/service overview page within the
 *   client area.
 *
 * Whichever option you choose, extra template variables are defined in the same
 * way. This demonstrates the use of the full replacement.
 *
 * Please Note: Using tabOverviewReplacementTemplate means you should display
 * the standard information such as pricing and billing details in your custom
 * template or they will not be visible to the end user.
 *
 * @param array $params common module parameters
 *
 * @see https://developers.whmcs.com/provisioning-modules/module-parameters/
 *
 * @return array
 */
function ugcc_ClientArea(array $params) {
    // Determine the requested action and set service call parameters based on
    // the action.
    $requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : '';

    if ($requestedAction == 'manage') {
        $serviceAction = 'get_usage';
        $templateFile = 'templates/manage.tpl';
    } else {
        $serviceAction = 'get_stats';
        $templateFile = 'templates/overview.tpl';
    }

    try {
        // Call the service's function based on the request action, using the
        // values provided by WHMCS in `$params`.
          $error = "";
        $status = "";
        $success = ugccapi_serverStatus($params['serverhttpprefix'] . "://" . $params['serverhostname'] . ":" . $params['serverport'], $params['serverusername'], $params['serverpassword'], $params['customfields']["SID"], $error);
        if (!$success) {
            $status = "Unable to connect to API";
        }
        else
        {
            if ($error === 'running')
            {
                $status = '<font color="green"><b>Running</b></font>';
            }
            if ($error === 'stopped')
            {
                $status = '<font color="red"><b>Stopped</b></font>';
            }
            if ($error === 'updating')
            {
                $status = '<font color="orange"><b>Updating</b></font>';
            }
        }
        $success = ugccapi_getServerDetails($params,$error);
        if (!$success) {
            $details = "Unable to connect to API";
        }
        $details = $error;
        $response = array();

        $extraVariable1 = 'abc';
        $extraVariable2 = '123';

        return array(
            'tabOverviewReplacementTemplate' => $templateFile,
            'templateVariables' => array(
                'extraVariable1' => $extraVariable1,
                'extraVariable2' => $extraVariable2,
                'clientid' => $params['clientid'],
                'ugccstatus' => $status,
                'ugccdetails' => $details,
            ),
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
                'ugcc',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
        );

        // In an error condition, display an error page.
        return array(
            'tabOverviewReplacementTemplate' => 'error.tpl',
            'templateVariables' => array(
                'usefulErrorHelper' => $e->getMessage(),
            ),
        );
    }
}
